<?php
defined('ABSPATH') or die("you do not have acces to this page!");



add_action('cmplz_notice_dpo_or_gdpr', 'cmplz_dpo_or_gdpr');
function cmplz_dpo_or_gdpr(){

    if (!cmplz_company_in_eu()){
        cmplz_notice(__("Your company is located outside the EU, so should appoint a GDPR representative in the EU.", 'complianz-gdpr'));
    } else {
        cmplz_notice(__("Your company is located in the EU, so you do not need to appoint a GDPR representative in the EU.", 'complianz-gdpr'));
    }

}

add_action('cmplz_notice_uses_social_media', 'cmplz_uses_social_media_notice');
function cmplz_uses_social_media_notice(){
    $social_media = cmplz_scan_detected_social_media();
    if ($social_media){
        $social_media = implode(', ', $social_media);
        cmplz_notice(sprintf(__("The scan found social media buttons or widgets for %s on your site, which means the answer should be yes", 'complianz-gdpr'), $social_media));
    }
}

add_action('cmplz_notice_purpose_personaldata', 'cmplz_purpose_personaldata');
function cmplz_purpose_personaldata(){
    $contact_forms = cmplz_site_uses_contact_forms();
    if ($contact_forms){
        cmplz_notice(__('The scan found forms on your site, which means answer should probably include "contact".', 'complianz-gdpr'));
    }
}

add_action('cmplz_notice_uses_thirdparty_services', 'cmplz_uses_thirdparty_services_notice');
function cmplz_uses_thirdparty_services_notice(){
    $thirdparty = cmplz_scan_detected_thirdparty_services();
    if ($thirdparty){
        $thirdparty = implode(', ', $thirdparty);
        cmplz_notice(sprintf(__("The scan found third party services for %s on your site, which means the answer should be yes", 'complianz-gdpr'), $thirdparty));
    }
}

add_action('cmplz_notice_purpose_personaldata', 'cmplz_purpose_personaldata_notice');
function cmplz_purpose_personaldata_notice(){
    if (cmplz_has_region('us') && COMPLIANZ()->cookie->uses_non_functional_cookies()){
        cmplz_notice(__("The cookie scan detected non-functional cookies on your site. According to the CaCPA, you are considered to 'Sell' personal data if you collect, share or sell personal data by any means. When a website uses non-functional cookies, it is collecting personal data.", 'complianz-gdpr'));
    }
}

add_action('cmplz_notice_data_disclosed_us', 'cmplz_data_disclosed_us');
function cmplz_data_disclosed_us(){

    if (COMPLIANZ()->cookie->uses_non_functional_cookies()) {
        cmplz_notice(__("The cookie scan detected non-functional cookies on your site. If these cookies were also used in the past 12 months, you should at least select the option 'Internet activity...'", 'complianz-gdpr'));
    }
}

add_action('cmplz_notice_data_sold_us', 'cmplz_data_sold_us');
function cmplz_data_sold_us(){

    if (COMPLIANZ()->cookie->uses_non_functional_cookies()) {
        cmplz_notice(__("The cookie scan detected non-functional cookies on your site. If these cookies were also used in the past 12 months, you should at least select the option 'Internet activity...'", 'complianz-gdpr'));
    }

}

add_action('cmplz_notice_no_cookies_used', 'cmplz_notice_no_cookies_used');
function cmplz_notice_no_cookies_used(){

    if (cmplz_get_value('uses_cookies')!=='yes') {
        cmplz_notice(__("You have indicated your site does not use cookies. If you're sure about this, you can skip this step", 'complianz-gdpr'),'warning');
    }

}

add_action('cmplz_notice_GTM_code', 'cmplz_notice_stats_non_functional');
add_action('cmplz_notice_UA_code', 'cmplz_notice_stats_non_functional');
add_action('cmplz_notice_matomo_site_id', 'cmplz_notice_stats_non_functional');

function cmplz_notice_stats_non_functional(){
    if (!cmplz_manual_stats_config_possible()) {
        cmplz_notice(__("You have selected options which indicate your statistics tracking needs a cookie banner. To enable Complianz to handle the statistics, you should remove your current statistics tracking, and configure it in Complianz", 'complianz-gdpr'),'warning');
    } else {
        cmplz_notice( __('If you add the ID for your statistics tool here, Complianz will configure your site for statistics tracking.', 'intro cookie usage', 'complianz-gdpr'));
    }
}

add_action('cmplz_notice_statistics_script', 'cmplz_notice_statistics_script');
function cmplz_notice_statistics_script(){

        cmplz_notice( __('You have indicated you use a statistics tool which tracks personal data. You can insert this script here so it only fires if the user consents to this.', 'intro cookie usage', 'complianz-gdpr'));

}


/*
 * For the cookie page and the US banner we need a link to the privacy statement.
 * In free, and in premium when the privacy statement is not enabled, we choose the WP privacy page. If it is not set, the user needs to create one.
 *
 *
 * */

add_action('cmplz_notice_missing_privacy_page', 'cmplz_notice_missing_privacy_page');
function cmplz_notice_missing_privacy_page(){
    $privacy_policy_exists = get_option('wp_page_for_privacy_policy') && get_post(get_option('wp_page_for_privacy_policy')) && get_post_status(get_option('wp_page_for_privacy_policy'))==='publish';
    if ((defined('cmplz_free') || cmplz_get_value('privacy-statement')!=='yes') && !$privacy_policy_exists) {
        cmplz_notice(sprintf(__("You do not have a privacy statement page selected, which is needed to configure your site. You can either let Complianz Privacy Suite premium handle it for you, or create one yourself and set it as the WordPress privacy page %shere%s", 'complianz-gdpr'),'<a href="'.admin_url('privacy.php').'">','</a>'),'warning');
    }

}

/*
 * Google Recaptcha is hard to integrate in a GDPR compliant manner, because the scripts will be deactivated.
 * A spammer only has to decline cookies, which prevents the recaptcha from blocking spam.
 *
 * */

add_action('cmplz_notice_thirdparty_services_on_site', 'cmplz_notice_thirdparty_services_on_site');
function cmplz_notice_thirdparty_services_on_site(){

        //only applies to opt-in
        if (!cmplz_has_region('eu')) return;

        $thirdparty = cmplz_scan_detected_thirdparty_services();
        if (is_array($thirdparty) && in_array('google-recaptcha', $thirdparty)) {
            cmplz_notice(__("The scan detected Google Recaptcha on your site. As the Recaptcha script will be blocked by default, it is recommended to use a honeypot spam protection instead.", 'complianz-gdpr'),'warning');
        }
}


add_filter('cmplz_default_value', 'cmplz_set_default', 10, 2);
function cmplz_set_default($value, $fieldname)
{
    if ($fieldname == 'purpose_personaldata') {
        if (cmplz_has_region('us') && COMPLIANZ()->cookie->uses_non_functional_cookies()) {
            //possibly not an array yet, when it's empty
            if (!is_array($value)) $value = array();
            $value['selling-data-thirdparty'] = 1;
            return $value;
        }
    }


    if ($fieldname == 'popup_background_color' || $fieldname == 'button_text_color') {
        $brand = cmplz_get_value('brand_color');
        if (!empty($brand)) return $brand;
    }

    if ($fieldname == 'purpose_personaldata') {
        $contact_forms = cmplz_site_uses_contact_forms();
        if ($contact_forms) {
            //possibly not an array yet, when it's empty
            if (!is_array($value)) $value = array();
            $value['contact'] = 1;
            return $value;
        }
    }

    if ($fieldname == 'dpo_or_gdpr') {
        if (!cmplz_company_in_eu()) return 'gdpr_rep';
    }

    if ($fieldname == 'country_company') {
        $country_code = substr(get_locale(),3,2);
        if (isset(COMPLIANZ()->config->countries[$country_code])) {
            $value = $country_code;
        }

    }

    if ($fieldname === 'uses_social_media'){
        $social_media = cmplz_scan_detected_social_media();
        if ($social_media) {
            return 'yes';
        }
    }

    if ($fieldname === 'socialmedia_on_site') {
        $social_media = cmplz_scan_detected_social_media();
        if ($social_media) {
            $current_social_media = array();
            foreach ($social_media as $key) {
                $current_social_media[$key] = 1;
            }
            return $current_social_media;
        }
    }

    if ($fieldname === 'uses_thirdparty_services'){
        $thirdparty = cmplz_scan_detected_thirdparty_services();
        if ($thirdparty) return 'yes';
    }

    if ($fieldname === 'thirdparty_services_on_site') {
        $thirdparty = cmplz_scan_detected_thirdparty_services();
        if ($thirdparty) {
            $current_thirdparty = array();
            foreach ($thirdparty as $key) {
                $current_thirdparty[$key] = 1;
            }

            return $current_thirdparty;
        }
    }

    if ($fieldname === 'data_disclosed_us' || $fieldname === 'data_sold_us'){
        if (COMPLIANZ()->cookie->uses_non_functional_cookies()) {
            //possibly not an array yet.
            if (!is_array($value)) $value = array();
            $value['internet'] = 1;
            return $value;
        }
    }

    return $value;
}
